@@ -1,6 +1,6 @@
 <?php
 /*
-V5.07 18 Dec 2008   (c) 2000-2008 John Lim. All rights reserved.
+V5.12 30 June 2011   (c) 2000-2011 John Lim. All rights reserved.
   Released under both BSD license and Lesser GPL library license. 
   Whenever there is any discrepancy between the two licenses, 
   the BSD license will take precedence.
@@ -49,8 +49,13 @@ class ADODB_oci8po extends ADODB_oci8 {
 		return ADODB_oci8::Prepare($sql,$cursor);
 	}
 	
+	function Execute($sql,$inputarr=false) 
+	{
+		return ADOConnection::Execute($sql,$inputarr);
+	}
+	
 	// emulate handling of parameters ? ?, replacing with :bind0 :bind1
-	function _query($sql,$inputarr)
+	function _query($sql,$inputarr=false)
 	{
 		if (is_array($inputarr)) {
 			$i = 0;
