@@ -1,6 +1,6 @@
 <?php
 /* 
-V5.07 18 Dec 2008   (c) 2000-2008 John Lim (jlim#natsoft.com). All rights reserved.
+V5.12 30 June 2011   (c) 2000-2011 John Lim (jlim#natsoft.com). All rights reserved.
   Released under both BSD license and Lesser GPL library license. 
   Whenever there is any discrepancy between the two licenses, 
   the BSD license will take precedence. 
@@ -134,10 +134,10 @@ class ADODB_ODBC_DB2 extends ADODB_odbc {
 		return $this->GetOne($this->identitySQL);
 	}
 	
-	function RowLock($tables,$where,$flds='1 as ignore')
+	function RowLock($tables,$where,$col='1 as adodbignore')
 	{
 		if ($this->_autocommit) $this->BeginTrans();
-		return $this->GetOne("select $flds from $tables where $where for update");
+		return $this->GetOne("select $col from $tables where $where for update");
 	}
 	
 	function MetaTables($ttype=false,$showSchema=false, $qtable="%", $qschema="%")
