@@ -1,6 +1,6 @@
 <?php
 /* 
-V5.05 11 July 2008   (c) 2000-2008 John Lim (jlim#natsoft.com). All rights reserved.
+V5.12 30 June 2011   (c) 2000-2011 John Lim (jlim#natsoft.com). All rights reserved.
   Released under both BSD license and Lesser GPL library license. 
   Whenever there is any discrepancy between the two licenses, 
   the BSD license will take precedence. See License.txt. 
@@ -15,6 +15,7 @@ V5.05 11 July 2008   (c) 2000-2008 John Lim (jlim#natsoft.com). All rights reser
 // security - hide paths
 if (!defined('ADODB_DIR')) die();
 
+
 class perf_oci8 extends ADODB_perf{
 
 	var $noShowIxora = 15; // if the sql for suspicious sql is taking too long, then disable ixora
@@ -71,6 +72,7 @@ AND    b.name = 'sorts (memory)'",
 		"select value from v\$sysstat where name='physical writes'"),
 	
 	'Data Cache',
+	
 		'data cache buffers' => array( 'DATAC',
 		"select a.value/b.value  from v\$parameter a, v\$parameter b 
 			where a.name = 'db_cache_size' and b.name= 'db_block_size'",
@@ -78,8 +80,24 @@ AND    b.name = 'sorts (memory)'",
 		'data cache blocksize' => array('DATAC',
 			"select value from v\$parameter where name='db_block_size'",
 			'' ),			
+	
 	'Memory Pools',
-		'data cache size' => array('DATAC',
+		'Mem Max Target (11g+)' => array( 'DATAC',
+		"select value from v\$parameter where name = 'memory_max_target'",
+			'The memory_max_size is the maximum value to which memory_target can be set.' ),
+	'Memory target (11g+)' => array( 'DATAC',
+		"select value from v\$parameter where name = 'memory_target'",
+			'If memory_target is defined then SGA and PGA targets are consolidated into one memory_target.' ),
+		'SGA Max Size' => array( 'DATAC',
+		"select nvl(value,0)/1024.0/1024 || 'M' from v\$parameter where name = 'sga_max_size'",
+			'The sga_max_size is the maximum value to which sga_target can be set.' ),
+	'SGA target' => array( 'DATAC',
+		"select nvl(value,0)/1024.0/1024 || 'M'  from v\$parameter where name = 'sga_target'",
+			'If sga_target is defined then data cache, shared, java and large pool size can be 0. This is because all these pools are consolidated into one sga_target.' ),
+	'PGA aggr target' => array( 'DATAC',
+		"select nvl(value,0)/1024.0/1024 || 'M' from v\$parameter where name = 'pga_aggregate_target'",
+			'If pga_aggregate_target is defined then this is the maximum memory that can be allocated for cursor operations such as sorts, group by, joins, merges. When in doubt, set it to 20% of sga_target.' ),
+	'data cache size' => array('DATAC',
 			"select value from v\$parameter where name = 'db_cache_size'",
 			'db_cache_size' ),
 		'shared pool size' => array('DATAC',
@@ -92,10 +110,7 @@ AND    b.name = 'sorts (memory)'",
 			"select value from v\$parameter where name='large_pool_size'",
 			'this pool is for large mem allocations (not because it is larger than shared pool), for MTS sessions, parallel queries, io buffers (large_pool_size) ' ),
 
-		'pga buffer size' => array('CACHE',			
-			"select value from v\$parameter where name='pga_aggregate_target'",
-			'program global area is private memory for sorting, and hash and bitmap merges - since oracle 9i (pga_aggregate_target)' ),
-
+		'dynamic memory usage' => array('CACHE', "select '-' from dual", '=DynMemoryUsage'),
 		
 		'Connections',
 		'current connections' => array('SESS',
@@ -128,8 +143,8 @@ AND    b.name = 'sorts (memory)'",
 			"select value from v\$parameter where name='sort_area_size'",
 			'max in-mem sort_area_size (per query), uses memory in pga' ),
 
-		'pga usage at peak' => array('RATIOU',
-		'=PGA','Mb utilization at peak transactions (requires Oracle 9i+)'),
+		/*'pga usage at peak' => array('RATIOU',
+		'=PGA','Mb utilization at peak transactions (requires Oracle 9i+)'),*/
 	'Transactions',
 		'rollback segments' => array('ROLLBACK',
 			"select count(*) from sys.v_\$rollstat",
@@ -159,25 +174,25 @@ having count(*) > 100)",'These are sql statements that should be using bind vari
 		'random page cost' => array('COST',
 			"select value from v\$parameter where name = 'optimizer_index_cost_adj'",
 			'=WarnPageCost'),
-		
+	'Waits',
+		'Recent wait events' => array('WAITS','select \'Top 5 events\' from dual','=TopRecentWaits'),
+//		'Historical wait SQL' => array('WAITS','select \'Last 2 days\' from dual','=TopHistoricalWaits'), -- requires AWR license
 	'Backup',
-		'Achivelog Mode' => array('BACKUP', 'select log_mode from v$database', 'To turn on archivelog:<br>
-	<pre>
-        SQLPLUS> connect sys as sysdba;
-        SQLPLUS> shutdown immediate;
-
-        SQLPLUS> startup mount exclusive;
-        SQLPLUS> alter database archivelog;
-        SQLPLUS> archive log start;
-        SQLPLUS> alter database open;
-</pre>'),
+		'Achivelog Mode' => array('BACKUP', 'select log_mode from v$database', '=LogMode'),
 	
 		'DBID' => array('BACKUP','select dbid from v$database','Primary key of database, used for recovery with an RMAN Recovery Catalog'),
 		'Archive Log Dest' => array('BACKUP', "SELECT NVL(v1.value,v2.value) 
 FROM v\$parameter v1, v\$parameter v2 WHERE v1.name='log_archive_dest' AND v2.name='log_archive_dest_10'", ''),
 	
-	'Flashback Area' => array('BACKUP', "select nvl(value,'Flashback Area not used') from v\$parameter where name=lower('DB_RECOVERY_FILE_DEST')", 'Flashback area is a folder where all backup data and logs can be stored and managed by Oracle. If Error: message displayed, then it is not in use.'),
-		'Control File Keep Time' => array('BACKUP', "select value from v\$parameter where name='control_file_record_keep_time'",'No of days to keep RMAN info in control file. I recommend it be set to x2 or x3 times the frequency of your full backup.'),
+		'Flashback Area' => array('BACKUP', "select nvl(value,'Flashback Area not used') from v\$parameter where name=lower('DB_RECOVERY_FILE_DEST')", 'Flashback area is a folder where all backup data and logs can be stored and managed by Oracle. If Error: message displayed, then it is not in use.'),
+	
+		'Flashback Usage' => array('BACKUP', "select nvl('-','Flashback Area not used') from v\$parameter where name=lower('DB_RECOVERY_FILE_DEST')", '=FlashUsage', 'Flashback area usage.'),
+		
+		'Control File Keep Time' => array('BACKUP', "select value from v\$parameter where name='control_file_record_keep_time'",'No of days to keep RMAN info in control file.  Recommended set to x2 or x3 times the frequency of your full backup.'),
+		'Recent RMAN Jobs' => array('BACKUP', "select '-' from dual", "=RMAN"),
+		
+		//		'Control File Keep Time' => array('BACKUP', "select value from v\$parameter where name='control_file_record_keep_time'",'No of days to keep RMAN info in control file. I recommend it be set to x2 or x3 times the frequency of your full backup.'),
+
 		false
 		
 	);
@@ -185,15 +200,108 @@ FROM v\$parameter v1, v\$parameter v2 WHERE v1.name='log_archive_dest' AND v2.na
 	
 	function perf_oci8(&$conn)
 	{
+	global $gSQLBlockRows;
+	
+		$gSQLBlockRows = 1000;
 		$savelog = $conn->LogSQL(false);	
 		$this->version = $conn->ServerInfo();
 		$conn->LogSQL($savelog);	
 		$this->conn = $conn;
 	}
 	
+	function LogMode()
+	{
+		$mode = $this->conn->GetOne("select log_mode from v\$database");
+		
+		if ($mode == 'ARCHIVELOG') return 'To turn off archivelog:<br>
+	<pre><font size=-2>
+        SQLPLUS> connect sys as sysdba;
+        SQLPLUS> shutdown immediate;
+
+        SQLPLUS> startup mount exclusive;
+        SQLPLUS> alter database noarchivelog;
+        SQLPLUS> alter database open;
+</font></pre>';
+		
+		return 'To turn on archivelog:<br>
+	<pre><font size=-2>
+        SQLPLUS> connect sys as sysdba;
+        SQLPLUS> shutdown immediate;
+
+        SQLPLUS> startup mount exclusive;
+        SQLPLUS> alter database archivelog;
+        SQLPLUS> archive log start;
+        SQLPLUS> alter database open;
+</font></pre>';
+	}
+	
+	function TopRecentWaits()
+	{
+		
+		$rs = $this->conn->Execute("select * from (   
+		select event, round(100*time_waited/(select sum(time_waited) from v\$system_event where wait_class <> 'Idle'),1) \"% Wait\",
+    total_waits,time_waited, average_wait,wait_class from v\$system_event where wait_class <> 'Idle' order by 2 desc
+	) where rownum <=5");
+		
+		$ret = rs2html($rs,false,false,false,false);		
+		return "&nbsp;<p>".$ret."&nbsp;</p>";
+		
+	}
+	
+	function TopHistoricalWaits()
+	{
+		$days = 2;
+		
+		$rs = $this->conn->Execute("select * from (   SELECT
+         b.wait_class,B.NAME,
+        round(sum(wait_time+TIME_WAITED)/1000000) waitsecs,
+        parsing_schema_name,
+        C.SQL_TEXT, a.sql_id
+FROM    V\$ACTIVE_SESSION_HISTORY A
+        join V\$EVENT_NAME B  on  A.EVENT# = B.EVENT#
+       join V\$SQLAREA C  on  A.SQL_ID = C.SQL_ID
+WHERE   A.SAMPLE_TIME BETWEEN sysdate-$days and sysdate
+       and parsing_schema_name not in ('SYS','SYSMAN','DBSNMP','SYSTEM')
+GROUP BY b.wait_class,parsing_schema_name,C.SQL_TEXT, B.NAME,A.sql_id
+order by 3 desc) where rownum <=10");
+		
+		$ret = rs2html($rs,false,false,false,false);		
+		return "&nbsp;<p>".$ret."&nbsp;</p>";
+		
+	}
+	
+	function RMAN()
+	{
+		$rs = $this->conn->Execute("select * from (select start_time, end_time, operation, status, mbytes_processed, output_device_type  
+			from V\$RMAN_STATUS order by start_time desc) where rownum <=10");
+		
+		$ret = rs2html($rs,false,false,false,false);		
+		return "&nbsp;<p>".$ret."&nbsp;</p>";
+		
+	}
+	function DynMemoryUsage()
+	{
+		if (@$this->version['version'] >= 11) {
+			$rs = $this->conn->Execute("select component, current_size/1024./1024 as \"CurrSize (M)\" from  V\$MEMORY_DYNAMIC_COMPONENTS");
+
+		} else
+			$rs = $this->conn->Execute("select name, round(bytes/1024./1024,2) as \"CurrSize (M)\" from  V\$sgainfo");
+
+			
+		$ret = rs2html($rs,false,false,false,false);		
+		return "&nbsp;<p>".$ret."&nbsp;</p>";
+	}
+	
+	function FlashUsage()
+	{
+        $rs = $this->conn->Execute("select * from  V\$FLASH_RECOVERY_AREA_USAGE");
+		$ret = rs2html($rs,false,false,false,false);		
+		return "&nbsp;<p>".$ret."&nbsp;</p>";
+	}
+	
 	function WarnPageCost($val)
 	{
-		if ($val == 100) $s = '<font color=red><b>Too High</b>. </font>';
+		if ($val == 100 && $this->version['version'] < 10) $s = '<font color=red><b>Too High</b>. </font>';
 		else $s = '';
 		
 		return $s.'Recommended is 20-50 for TP, and 50 for data warehouses. Default is 100. See <a href=http://www.dba-oracle.com/oracle_tips_cost_adj.htm>optimizer_index_cost_adj</a>. ';
@@ -201,7 +309,7 @@ FROM v\$parameter v1, v\$parameter v2 WHERE v1.name='log_archive_dest' AND v2.na
 	
 	function WarnIndexCost($val)
 	{
-		if ($val == 0) $s = '<font color=red><b>Too Low</b>. </font>';
+		if ($val == 0 && $this->version['version'] < 10) $s = '<font color=red><b>Too Low</b>. </font>';
 		else $s = '';
 		
 		return $s.'Percentage of indexed data blocks expected in the cache.
@@ -210,23 +318,35 @@ FROM v\$parameter v1, v\$parameter v2 WHERE v1.name='log_archive_dest' AND v2.na
 		}
 	
 	function PGA()
+	{	
+		
+		//if ($this->version['version'] < 9) return 'Oracle 9i or later required';
+	}
+	function PGA_Advice()
 	{
-		if ($this->version['version'] < 9) return 'Oracle 9i or later required';
+		$t = "<h3>PGA Advice Estimate</h3>";
+		if ($this->version['version'] < 9) return $t.'Oracle 9i or later required';
 		
-		$rs = $this->conn->Execute("select a.mb,a.targ as pga_size_pct,a.pct from 
-	   (select round(pga_target_for_estimate/1024.0/1024.0,0) Mb,
-	   	   pga_target_factor targ,estd_pga_cache_hit_percentage pct,rownum as r 
-	   	   from v\$pga_target_advice) a left join
-	   (select round(pga_target_for_estimate/1024.0/1024.0,0) Mb,
-	   	   pga_target_factor targ,estd_pga_cache_hit_percentage pct,rownum as r 
-	   	   from v\$pga_target_advice) b on 
-	  a.r = b.r+1 where 
-  		b.pct < 100");
-		if (!$rs) return "Only in 9i or later";
-		$rs->Close();
-		if ($rs->EOF) return "PGA could be too big";
+		$rs = $this->conn->Execute('select a.MB,
+			case when a.targ = 1 then \'<<= Current \' 
+			when a.targ < 1  or a.pct <= b.pct then null 
+			else 
+			\'- BETTER than Current by \'||round(a.pct/b.pct*100-100,2)||\'%\' end as "Percent Improved",
+	a.targ as  "PGA Size Factor",a.pct "% Perf"
+	from 
+       (select round(pga_target_for_estimate/1024.0/1024.0,0) MB,
+              pga_target_factor targ,estd_pga_cache_hit_percentage pct,rownum as r 
+              from v$pga_target_advice) a left join
+       (select round(pga_target_for_estimate/1024.0/1024.0,0) MB,
+              pga_target_factor targ,estd_pga_cache_hit_percentage pct,rownum as r 
+              from v$pga_target_advice) b on 
+      a.r = b.r+1 where 
+          b.pct < 100');
+		if (!$rs) return $t."Only in 9i or later";
+	//	$rs->Close();
+		if ($rs->EOF) return $t."PGA could be too big";
 		
-		return reset($rs->fields);
+		return $t.rs2html($rs,false,false,true,false);
 	}
 	
 	function Explain($sql,$partial=false) 
@@ -314,28 +434,31 @@ CONNECT BY prior id=parent_id and statement_id='$id'");
 		if ($this->version['version'] < 9) return 'Oracle 9i or later required';
 		
 		 $rs = $this->conn->Execute("
-select  a.size_for_estimate as cache_mb_estimate,
-	case when a.size_factor=1 then 
-   		'&lt;&lt;= current'
-	 when a.estd_physical_read_factor-b.estd_physical_read_factor > 0 and a.estd_physical_read_factor<1 then
-		'- BETTER - '
-	else ' ' end as currsize, 
-   a.estd_physical_read_factor-b.estd_physical_read_factor as best_when_0
-   from (select size_for_estimate,size_factor,estd_physical_read_factor,rownum  r from v\$db_cache_advice) a , 
-   (select size_for_estimate,size_factor,estd_physical_read_factor,rownum r from v\$db_cache_advice) b where a.r = b.r-1");
+select  a.name Buffer_Pool, b.size_for_estimate as cache_mb_estimate, 
+	case when b.size_factor=1 then 
+   		'&lt;&lt;= Current'
+	 when a.estd_physical_read_factor-b.estd_physical_read_factor > 0.001 and b.estd_physical_read_factor<1 then
+		'- BETTER than current by ' || round((1-b.estd_physical_read_factor)/b.estd_physical_read_factor*100,2) || '%'
+	else ' ' end as RATING, 
+   b.estd_physical_read_factor \"Phys. Reads Factor\",
+   round((a.estd_physical_read_factor-b.estd_physical_read_factor)/b.estd_physical_read_factor*100,2) as \"% Improve\"
+   from (select size_for_estimate,size_factor,estd_physical_read_factor,rownum  r,name from v\$db_cache_advice order by name,1) a , 
+   (select size_for_estimate,size_factor,estd_physical_read_factor,rownum r,name from v\$db_cache_advice order by name,1) b 
+   where a.r = b.r-1 and a.name = b.name
+  ");
 		if (!$rs) return false;
 		
 		/*
 		The v$db_cache_advice utility show the marginal changes in physical data block reads for different sizes of db_cache_size
 		*/
-		$s = "<h3>Data Cache Estimate</h3>";
+		$s = "<h3>Data Cache Advice Estimate</h3>";
 		if ($rs->EOF) {
 			$s .= "<p>Cache that is 50% of current size is still too big</p>";
 		} else {
-			$s .= "Ideal size of Data Cache is when \"best_when_0\" changes from a positive number and becomes zero.";
+			$s .= "Ideal size of Data Cache is when %BETTER gets close to zero.";
 			$s .= rs2html($rs,false,false,false,false);
 		}
-		return $s;
+		return $s.$this->PGA_Advice();
 	}
 	
 	/*
@@ -547,11 +670,12 @@ BEGIN
 	LOOP
 	  cnt := cnt + 1;
 	  DELETE FROM $perf_table WHERE ROWID=rec.rr;
-	  IF cnt = 10000 THEN
+	  IF cnt = 1000 THEN
 	  	COMMIT;
 		cnt := 0;
 	  END IF;
 	END LOOP;
+	commit;
 END;";
 
 		$ok = $this->conn->Execute($sql);
